from kubernetes import config, client
from pick import pick
import sys, re, math

class Kube():
    def selectContext():
        contexts, activeContext = config.list_kube_config_contexts()
        if not contexts:
            print("No contexts found - please set up your kubeconfig file")
            sys.exit(2)
        if len(contexts) == 1:
            cluster = activeContext["name"]
        else:
            contextNames = [context['name'] for context in contexts]
            active = contextNames.index(activeContext["name"])
            cluster, _ = pick(contextNames, "Select cluster", default_index=active)

        return client.CoreV1Api(api_client=config.new_client_from_config(context=cluster)), cluster

class Parsers():
    def parseMemoryResourceValue(value):
        match = re.match(r'^([0-9]+)(E|Ei|P|Pi|T|Ti|G|g|Gi|M|Mi|m|K|k|Ki){0,1}$', value)
        if match is None:
            return int(value)
        amount = match.group(1)
        eom = match.group(2).capitalize()
        
        calc = {
            "Ki": math.pow(1024,1),
            "K": 1000,
            "Mi": math.pow(1024,2),
            "M": 1000000,
            "Gi": math.pow(1024,3),
            "G": 1000000000
        }

        return int(amount) * calc.get(eom)

    def parseCpuResourceValue(value):
        match = re.match(r'^([0-9]+)m$', value)
        if match is not None:
            return int(match.group(1))
        return int(value) * 1000


import sys, getopt, urllib3
from kubernetes import client, config
from kubernetes.client.rest import ApiException
from progress.bar import Bar
from colorama import Fore, Style
from .entities import NodeData
from .helpers import Kube, Parsers
import adal
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

def main():
    argv = sys.argv[1:]
    verbose = False
    try:
        opts, args = getopt.getopt(argv, "hv", ["verbose"])
    except getopt.GetoptError:
        print("{} [-v]".format(__file__))
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("{} -v (optional) - Lists requests for each pod on the nodes".format(__file__))
            sys.exit(1)
        elif opt in ("-v", "--verbose"):
            verbose = True
    api, contextName = Kube.selectContext()
    print(Fore.GREEN + "Active kube context: {}".format(contextName) + Style.RESET_ALL)
    try:
        allData = []
        nodes = api.list_node(label_selector='!node-role.kubernetes.io/master')
        for node in nodes.items:            
            nodeName = node.metadata.name
            nodeData = NodeData(nodeName, node.status.capacity)
            pod_templates = api.list_pod_for_all_namespaces(field_selector='spec.nodeName=%s,status.phase!=Failed,status.phase!=Succeeded' % nodeName)
            for template in pod_templates.items:
                name = template.metadata.name
                requests = parseResourceRequestsForAllContainers(template.spec.containers)
                nodeData.addCpuRequest(name, requests["cpu"])
                nodeData.addMemRequest(name, requests["mem"])

                limits = parseResourceLimitsForAllContainers(template.spec.containers)
                nodeData.addCpuLimit(name, limits["cpu"])

            allData.append(nodeData)

        printResourceReport(allData, verbose)
    except ApiException as e:
        print("Error when attempting to read node data: %s\n" % e)

def printResourceReport(data, verbose : bool):
        for node in data:
            print(Fore.CYAN + "Node {:20}".format(node.name) + Style.RESET_ALL)

            barCpu = Bar("Requested CPU   ", max=node.cpuCapacity , suffix='%(percent)d%%', fill=Fore.YELLOW + "#" + Style.RESET_ALL)
            barCpu.goto(node.totalCpuRequests)
            barCpu.finish()
            barCpu = Bar("Limits CPU      ", max=node.cpuCapacity , suffix='%(percent)d%%', fill=Fore.YELLOW + "#" + Style.RESET_ALL)
            barCpu.goto(node.totalCpuLimits)
            barCpu.finish()
            barMem = Bar("Requested memory", max=node.memCapacity , suffix='%(percent)d%%', fill=Fore.YELLOW + "#" + Style.RESET_ALL)
            barMem.goto(node.totalMemRequests)
            barMem.finish()
            
            print()
            
            if verbose:
                for pod in node.cpuRequests.keys():
                    if node.cpuRequests[pod] == 0:
                        continue
                    print("\t{:50}{:>5}".format(pod, node.cpuRequests[pod]))
            
        print(Fore.CYAN + "\nTotal cluster utilization" + Style.RESET_ALL)
        barTotalCpu = Bar("Requested CPU   ", max=NodeData.totalCpuCapacity, suffix='%(percent)d%%', fill=Fore.YELLOW + "#" + Style.RESET_ALL)
        barTotalCpu.goto(NodeData.totalCpuRequests)
        barTotalCpu.finish()
        barTotalMem = Bar("Requested Memory", max=NodeData.totalMemCapacity, suffix='%(percent)d%%', fill=Fore.YELLOW + "#" + Style.RESET_ALL)
        barTotalMem.goto(NodeData.totalMemRequests)
        barTotalMem.finish()

def parseResourceRequestsForAllContainers(containers):
    cpuRequests = 0
    memRequests = 0

    for container in containers:    
        if container.resources is None or container.resources.requests is None:
            continue
    
        requests = container.resources.requests
        if "cpu" in requests:
            cpuRequests += Parsers.parseCpuResourceValue(requests["cpu"])
        if "memory" in requests:
            memRequests += Parsers.parseMemoryResourceValue(requests["memory"])
    
    return {"cpu": cpuRequests, "mem": memRequests}

def parseResourceLimitsForAllContainers(containers):
    cpuLimits = 0
    memLimits = 0

    for container in containers:    
        if container.resources is None or container.resources.limits is None:
            continue
    
        limits = container.resources.limits
        if "cpu" in limits:
            cpuLimits += Parsers.parseCpuResourceValue(limits["cpu"])
        if "memory" in limits:
            memLimits += Parsers.parseMemoryResourceValue(limits["memory"])
    
    return {"cpu": cpuLimits, "mem": memLimits}

if __name__ == "__main__":
    main()



from .helpers import Parsers

class NodeData:
    totalCpuRequests = 0
    totalCpuLimits = 0
    totalMemRequests = 0
    totalCpuCapacity = 0
    totalMemCapacity = 0

    def __init__(self, nodename, capacity):
        self.cpuRequests = {}
        self.memRequests = {}
        self.cpuLimits = {}
        self.name = nodename
        self.cpuCapacity = int(capacity["cpu"]) * 1000
        self.memCapacity = Parsers.parseMemoryResourceValue(capacity["memory"])
        self.totalCpuRequests = 0
        self.totalMemRequests = 0
        self.totalCpuLimits = 0

        NodeData.totalCpuCapacity += self.cpuCapacity
        NodeData.totalMemCapacity += self.memCapacity

    def addCpuRequest(self, podName, cpuRequest):
        NodeData.totalCpuRequests += cpuRequest
        self.totalCpuRequests += cpuRequest
        self.cpuRequests[podName] = cpuRequest
    
    def addCpuLimit(self, podName, cpuLimit):
        NodeData.totalCpuLimits += cpuLimit
        self.totalCpuLimits += cpuLimit
        self.cpuLimits[podName] = cpuLimit

    def addMemRequest(self, podName, memRequest):
        NodeData.totalMemRequests += memRequest
        self.totalMemRequests += memRequest
        self.memRequests[podName] = memRequest



import setuptools

setuptools.setup(
    name="kuberesources",
    version="1.1",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'kresources=kuberesources.__main__:main'
        ]
    },
    author="Frode Hus",
    author_email="frode.hus@outlook.com",
    description="Simple utility for getting an overview of set requests and limits in a kubernetes cluster",
    url="https://github.com/frodehus/kuberesources",
    python_requires='>=3.6',
    install_requires=[
        "kubernetes",
        "pick",
        "adal",
        "colorama",
        "progress"
    ],
    classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
)
